import pymongo
from bson import ObjectId
from fastapi import APIRouter, Depends, Request
from motor.motor_asyncio import AsyncIOMotorDatabase
import crud, schemas
from api import deps
from core.config import settings

from db import get_database
from db.ckdb import CKDrive, get_ck_db
from schemas.project import ProjectCreate
from utils import casbin_enforcer

router = APIRouter()


@router.post("/create")
async def create(
        request: Request,
        data_in: ProjectCreate,
        db: AsyncIOMotorDatabase = Depends(get_database),
        current_user: schemas.UserDB = Depends(deps.get_current_user)
) -> schemas.Msg:
    """创建项目"""
    try:
        res_project = await crud.project.create(db, data_in, current_user=request.user)
    except pymongo.errors.DuplicateKeyError:
        return schemas.Msg(code=-1, msg='项目名已存在', data='项目名已存在')

    folder = schemas.FolderCreate(
        name='未分组',
        project_id=res_project.inserted_id,
        cat='kanban',
        pid=res_project.inserted_id,
    )
    await crud.folder.create(db, folder, user_id=current_user.id)
    folder = schemas.FolderCreate(
        name='共享给我的',
        project_id=res_project.inserted_id,
        cat='kanban',
        pid=res_project.inserted_id,
    )
    await crud.folder.create(db, folder, user_id=current_user.id)

    # 创建全部数据权限
    data_auth = schemas.DataAuthCreate(name='全部', data=['*'])
    await crud.data_auth.create(db, data_auth, data_in.game)

    # 新建项目管理员权限
    role_name = f'{data_in.game}_admin'
    role_dom = data_in.game
    casbin_enforcer.add_policy(role_name, role_dom, '*', '*')
    await crud.authority.create(db, 'p', role_name, role_dom, '*', '*')
    # 添加角色
    await crud.authority.create(db, 'g', settings.SUPERUSER_NAME, role_name, '*', '*', role_name='项目管理员', game=role_dom)

    return schemas.Msg(code=0, msg='创建成功')


@router.get("/")
async def read_project(request: Request,
                       db: AsyncIOMotorDatabase = Depends(get_database),
                       current_user: schemas.UserDB = Depends(deps.get_current_user)
                       ):
    """查看自己拥有的项目"""
    res = await crud.project.read_project(db, username=request.user.username)

    return schemas.Msg(code=0, msg='ok', data=res)


@router.get("/detail")
async def read_project(request: Request,
                       game: str,
                       db: AsyncIOMotorDatabase = Depends(get_database),
                       ck: CKDrive = Depends(get_ck_db),
                       current_user: schemas.UserDB = Depends(deps.get_current_user)
                       ):
    """查看项目信息"""
    res = await crud.project.read_project(db, username=request.user.username, game=game)
    if res:
        res = res[0]
    event_count = await ck.count(game, 'event')
    user_count = await ck.count(game, 'user_view')
    event_type_count = await ck.distinct_count(game, 'event', '#event_name')
    event_attr_count = await ck.field_count(db=game, tb='event')
    user_attr_count = await ck.field_count(db=game, tb='user_view')

    res['event_count'] = event_count
    res['user_count'] = user_count
    res['event_type_count'] = event_type_count
    res['event_attr_count'] = event_attr_count
    res['user_attr_count'] = user_attr_count
    return schemas.Msg(code=0, msg='ok', data=res)


@router.post("/rename")
async def rename_project(request: Request,
                         game: str,
                         data_in: schemas.ProjectRename,
                         db: AsyncIOMotorDatabase = Depends(get_database),
                         current_user: schemas.UserDB = Depends(deps.get_current_user)
                         ):
    """修改项目名"""
    try:
        res = await crud.project.rename(db, data_in)
    except pymongo.errors.DuplicateKeyError:
        return schemas.Msg(code=-1, msg='项目名已存在')
    return schemas.Msg(code=0, msg='ok', data=res)


# todo 要修改
@router.post("/add_members")
async def add_members(request: Request,
                      game: str,
                      data_in: schemas.ProjectAddMember,
                      db: AsyncIOMotorDatabase = Depends(get_database),
                      current_user: schemas.UserDB = Depends(deps.get_current_user)
                      ):
    """项目添加成员"""

    for item in data_in.members:
        casbin_enforcer.add_grouping_policy(item.username, item.role_name, game)
        # 设置数据权限
        await crud.authority.set_data_auth(db,
                                           schemas.DataAuthSet(username=item.username, data_auth_id=item.data_auth_id),
                                           game)

    await crud.project.add_members(db, schemas.ProjectMember(project_id=data_in.project_id,
                                                             members=[item.username for item in data_in.members]))

    return schemas.Msg(code=0, msg='ok', data=data_in)


@router.post("/edit_member")
async def edit_member(request: Request,
                      game: str,
                      db: AsyncIOMotorDatabase = Depends(get_database),
                      current_user: schemas.UserDB = Depends(deps.get_current_user)
                      ):
    """编辑成员权限  角色和数据"""

    pass
    return schemas.Msg(code=0, msg='ok', )


#
#
@router.get("/members")
async def members(request: Request,
                  game: str,
                  db: AsyncIOMotorDatabase = Depends(get_database),
                  current_user: schemas.UserDB = Depends(deps.get_current_user)
                  ):
    """查看项目成员"""
    roles = await crud.authority.find_many(db, ptype='g', v2=game)
    data = dict()
    data_auth_ids = set()
    for item in roles:
        data[item['v0']] = {'name': item['v0'],
                            'role': item['v1'],
                            'data_auth_id': item.get('data_auth_id')
                            }
        if item.get('data_auth_id'):
            data_auth_ids.add(ObjectId(item.get('data_auth_id')))

    data_auth = {str(m['_id']): m['title'] async for m in
                 crud.data_auth.find(db, {'_id': {'$in': list(data_auth_ids)}}, {'title': 1})}

    data['root'] = {'name': 'root', 'role': '超级管理员'}
    users = await crud.user.get_by_users(db, name={'$in': list(data.keys())})
    res = []
    for user in users.data:
        res.append({
            **user.dict(),
            'role': data[user.name]['role'],
            'data_auth': data_auth.get(data[user.name].get('data_auth_id'), '全部事件')
        })

    return schemas.Msg(code=0, msg='ok', data=res)


@router.post("/del_member")
async def members(request: Request,
                  game: str,
                  data_in: schemas.ProjectDelMember,
                  db: AsyncIOMotorDatabase = Depends(get_database),
                  current_user: schemas.UserDB = Depends(deps.get_current_user)
                  ):
    """删除项目成员"""
    casbin_enforcer.delete_roles_for_user_in_domain(data_in.username,data_in.role,game)
    await crud.project.del_members(db, data_in)
    await crud.authority.delete(db, ptype='g', v2=game, v0=data_in.username)
    return schemas.Msg(code=0, msg='ok')


@router.post("/kanban")
async def read_kanban(
        data_in: schemas.ProjectKanban,
        db: AsyncIOMotorDatabase = Depends(get_database),
        current_user: schemas.UserDB = Depends(deps.get_current_user)
):
    """获取自己的看板"""
    res = {'kanban': [], 'spaces': []}
    # 我的看板
    kanban = await crud.folder.read_folder(db, project_id=data_in.id, user_id=current_user.id, cat='kanban')

    for item in kanban:
        dashboards = await crud.dashboard.find_many(db, pid=item['_id'])
        res['kanban'].append({
            'name': item['name'],
            'children': [],
            '_id': item['_id']
        })
        for d in dashboards:
            res['kanban'][-1]['children'].append({
                'name': d['name'],
                '_id': item['_id']
            })

    # 我的空间
    where = {
        'project_id': data_in.id,
        'members._id': current_user.id
        # '$or': [{'rw_members': current_user.id}, {'r_members': current_user.id}]
    }
    spaces = await crud.space.find_many(db, **where)
    # 空间 文件夹  看板
    for item in spaces:
        res['spaces'].append({
            'name': item['name'],
            'children': [],
            '_id': item['_id']
        })
        res['spaces'][-1]['authority'] = 'rw' if current_user.id in item['members'] else 'r'

        for f in await crud.folder.find_many(db, pid=item['_id']):
            res['spaces'][-1]['children'].append({
                'name': f['name'],
                '_id': f['_id'],
                'children': [],
                'isFolder': True
            })

            for d in await crud.dashboard.find_many(db, pid=f['_id']):
                res['spaces'][-1]['children'][-1]['children'].append({
                    'name': d['name'],
                    '_id': d['_id']
                })

        # 空间 看板
        for d in await crud.dashboard.find_many(db, pid=item['_id']):
            res['spaces'][-1]['children'].append({
                'name': d['name'],
                '_id': d['_id'],
                'isFolder': False
            })

    return schemas.Msg(code=0, msg='ok', data=res)
